/* -*- C -*- 
c     Program cavity
c     Original Coded By K.Yanagi (Fortran)
c     Arranged By H. Kogo   1991. 6.29 (Fortran)
c              By T. Nemoto 1993.10    Version 4.0 (Fortran)
c                           1994.10    Version 4.1 (Fortran)
c                           1994.11    Version 4.5 (Fortran)
c                           1997.12    Version 5.0 (C)
c                           2002.12    Version 5.1 (C)
c                           2006.02    Version 5.4 (C)
c                           2008.08    Version 5.5.4 (C)
*/

#include "cavity.h"
#include <stdlib.h>
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

static int CavMainLoop(void);
static int PlotVDW(int nSec,int iZNum);
static void setpl(void);
static void CheckRange(void);
static double PlVol(void);
static int AutoExpandRange(int nSec,int l,double xp,double yp);
static int AutoExpandRange2(int nSec,int l,
			    double x,double y,double r, double st, double ed);
static double zsec;

void error(char *mess) {
  fprintf(stderr,"#Error:%s\n",mess);
  CavPlotComment(mess);
  iErr=1;
  CavTrailer();
  exit(1);
}

int main(int argc,char **argv) {
  int iv;

  fprintf(stderr,"Cavity version %s\n",VERSION);
  /*** read file , etc... ***/
  if (argc==2) {
    CavInit(argv[1]);
  } else if (argc==1) {
    CavInit("cav.cnt");
  } else {
    fprintf(stderr,"Usage: %s [<control-file>]\n",argv[0]);
    exit(1);
  }
  /*** iview loop ***/
  iv=1;
  if (iView==0) {
    /*      ... Mesh Mode */
    iv=3;
    iView=3;
  }
  CavPlotMeshDirection(iView);

  do {
    CavPlotMeshBegin(iView);
    /*     ... set axis, etc.  */
    cavin2();
    CavPlotMatrix();
    /*** cavity Main ***/
    /*     ... plot one view*/
    CavMainLoop();
    CavPlotMeshEnd();
    iView--;
  } while(--iv>0);
  /*** write volume, etc... ***/
  CavTrailer();
  return 0;
}

/********************************************************
      SUBROUTINE PRJCTN
********************************************************/

void Prjctn(void) {
  int i;
  CavPlotPen(4);
  if (radi>0) {
    for(i=0;i<nPlt;i++){
      int ipl=iPlt[i];
      CavPlotArc(xyz[ipl][kx],xyz[ipl][ky],xyz[ipl][kz],radi,0,360,natm[ipl]);
    }
  }

  for(i=0;i<nBond;i++){
    int ips,ipe;
    ips=iPs[i];
    ipe=iPe[i];
    if (bExclFlag[ips]!=0 && bExclFlag[ipe]!=0) {
      /*    *** exclusive -- in Cavity *** */
      CavPlotPen(3);
    } else {
      /* **** Not Exclusive **** */
      CavPlotPen(2);
    }
    CavPlotLine(xyz[ips][kx],xyz[ips][ky],xyz[ips][kz],
	   xyz[ipe][kx],xyz[ipe][ky],xyz[ipe][kz],
	   natm[ips],natm[ipe]);
  }
}

static int CavMainLoop(void) {
  int iRetry;
  int iZNum,nSec;

  CavPlotPen(1);
  /*     *** Title *** */
  CavPlotTitle(title);
  if (iPlot!=0) Prjctn();
  CavPlotPen(1);
  CavPlotFlush();

  /*     *** Main Loop *** */
  CavPlotComment("Cavity Start");
  iRetry=0;

  vol=0;
  iZNum=0;
  /*     *** Main Loop *** */
  do {
    CheckRange();
    nSec=(zmax-zmin)/divz;
    zsec=zmin+divz*iZNum;
    sprintf(lbuf,"Z = %7.3f (%4d/%4d) IView=%1d",zsec,iZNum,nSec,iView);
    CavPlotComment(lbuf);
    CavPlotFlush();
    setpl();
    if (PlotVDW(nSec,iZNum)==0) continue;
    plvol=PlVol();
    vol+=plvol;
    sprintf(lbuf,"Volume/Plane =%7.3f",plvol);
    CavPlotComment(lbuf);
    CavPlotFlush();
    iZNum++;
  } while (iZNum<=nSec);
  return 1;
}


/*********************************************
 findcross (x0,y0,r0, x1,y1,r1, *ang1, *ang2
 return:  0: no contact
          1: normal end 
          2: circle 0 is in the circle 1
         -1: circle 1 is in the circle 0
**********************************************/

static int findcross(double x0, double y0, double r0,
	      double x1, double y1, double r1,
	      double *ang0, double *ang1){
  double R,R2,R22,C,X0,Y0,X1,Y1,rtx,rty,t,a0,a1,av,ax,ay,r02,r12,x12,y12;
  x1-=x0;
  y1-=y0;
  r12=r1*r1;
  r02=r0*r0;
  x12=x1*x1;
  y12=y1*y1;
  R2=x12+y12;
  R=sqrt(R2);
  if (r1>R+r0) {
    *ang0=0.0;
    *ang1=TWOPI;
    return 2;
  }
  if (r0>R+r1) return -1;
  R22=0.5/R2;
  C=r12-r02-R2;
  rtx=-C*C*y12+4*R2*y12*r02;
  rty=-C*C*x12+4*R2*x12*r02;
  if (rtx<0 || rty<0) return 0;
  if (rtx<1.0E-5 && rty<1.0E-5) return 0;
  
  Y0=R22*(-y1*C+sqrt(rty));
  Y1=R22*(-y1*C-sqrt(rty));
  X0=R22*(-x1*C+sqrt(rtx));
  X1=R22*(-x1*C-sqrt(rtx));
  if (fabs((X1-x1)*(X1-x1)+(Y1-y1)*(Y1-y1)-r12)>1.0E-5) {
    t=Y1;
    Y1=Y0;
    Y0=t;
  }
  
  a0=atan2(Y0,X0);
  a1=atan2(Y1,X1);
  if (a0>a1) {
    t=a1;
    a1=a0;
    a0=t;
  }
  av=0.5*(a0+a1);
  ax=r0*cos(av)-x1;
  ay=r0*sin(av)-y1;
  if (ax*ax+ay*ay>r12) {
    *ang0=a1;
    *ang1=a0+TWOPI;
  } else {
    *ang0=a0;
    *ang1=a1;
  }
  return 1;
}

int findcrossX1(double x0, double r0,
	      double x1, double *ang0, double *ang1){
  double x=x1-x0;
  if (r0*r0<=x*x) return 0;
  *ang0=-acos((x1-x0)/r0);
  *ang1=acos((x1-x0)/r0);
  return 1;
}

int findcrossY1(double y0, double r0,
	      double y1, double *ang0, double *ang1){
  double y=y1-y0;
  double as;
  if (r0*r0<=y*y) return 0;
  as=asin((y1-y0)/r0);
  *ang0=as;
  *ang1=PI-as;
  return 1;
}

int findcrossX2(double x0, double y0, double r02,
	      double x1, double *Y0, double *Y1){
  double x2=(x1-x0)*(x1-x0);
  if (r02<=x2) return 0;
  *Y0=y0-sqrt(r02-x2);
  *Y1=y0+sqrt(r02-x2);
  return 1;
}

/************************************/
static int SortRange(int *countp, double *st, double *ed) {
  int j,k,count,count1;
  double t;
  count=*countp;
  for(j=0;j<count-1;j++){
    for(k=j+1;k<count;k++){
      if (st[j]>st[k]) {
	t=st[j];
	st[j]=st[k];
	st[k]=t;
	t=ed[j];
	ed[j]=ed[k];
	ed[k]=t;
      }
    }
  }

  count1=0;
  for(j=1;j<count;j++) {
    if (ed[count1]<st[j]) {
      count1++;
      st[count1]=st[j];
      ed[count1]=ed[j];
    } else if (/* st[count1]<=st[j] && */ ed[count1]<ed[j]) {
      ed[count1]=ed[j];
    }
  }
  *countp=count1+1;
  return *countp;
}

static int CheckArc(int *countp, double *arcs, double *arce) {
  int j,arcc,arcc1;
  arcc=arcc1=*countp;
  for(j=0;j<arcc1;j++) {
    while (arcs[j]<0) {
      arcs[j]+=TWOPI;
      arce[j]+=TWOPI;
    }
    while (arcs[j]>TWOPI) {
      arcs[j]-=TWOPI;
      arce[j]-=TWOPI;
    }
    if (arce[j]>TWOPI) {
      arcs[arcc]=0.0;
      arce[arcc]=arce[j]-TWOPI;
      arce[j]=TWOPI;
      arcc++;
    }
  }
  *countp=arcc;
  SortRange(countp, arcs, arce);
  return *countp;
}

double FindSection(double *aminp, double *amaxp, double X0,
		   double *X,double *Y, double SX, double SY,
		   double *st, double *ed,int *countp) {
  int count=0;
  int i;
  double a0,a1;
  double len=0.0;
  double amin,amax;
  amin=*aminp;
  amax=*amaxp;
  if (sr2>0) {
    if (findcrossX2(SX,SY,sr2,X0,&a0,&a1)) {
      if (a0>amin) amin=a0;
      if (a1<amax) amax=a1;
    } else return 0.0; /* No cross */
  } 
  for(i=0;i<kPlot;i++) {
    if (findcrossX2(X[i],Y[i],r2[i],X0,&a0,&a1)) {
      if (a1<amin || a0>amax) continue;
      st[count]=a0;
      ed[count]=a1;
      count++;
    }
  }
  SortRange(&count,st,ed);
  st[count]=amax;
  *countp=count;
  *aminp=amin;
  *amaxp=amax;
  for(i=0;i<=count;i++) {
    if (st[i]>amin) {
      len+=st[i]-amin;
      amin=ed[i];
    } else if (amin<ed[i]) {
      amin=ed[i];
    }
  }
  return len;
}

double DrawSection(int vertical, double amin, double amax, double X0,
		   double *X,double *Y, double SX, double SY,
		   double *st, double *ed, char *name) {
  int count,i;
  double len;
  len=FindSection(&amin,&amax,X0,X,Y,SX,SY,st,ed,&count);
  for(i=0;i<=count;i++) {
    if (st[i]>amin) {
      CavPlotPen(1);
      CavPlotColor(iCol[0]);
      if (vertical!=0) {
	CavPlotLine(X0,amin,zsec,X0,st[i],zsec,name,"");
      } else {
	CavPlotLine(amin,X0,zsec,st[i],X0,zsec,name,"");
      }
      amin=ed[i];
    } else if (amin<ed[i]) {
      amin=ed[i];
    }
  }
  return len;
}

int PlotVDW(int nSec,int iZNum) {
  double arcs[MaxAtm],arce[MaxAtm];
  int arcc;
  int i,j,k;

  if (sr2<0) return 1; /* Limit Sphere */

  for(i=0;i<nPlot;i++){
    iUsePl[i]=0;
  }

  for(i=0;i<=kPlot;i++){
    double arcx,arcy,arcr,arcr2;
    double a0,a1;

    if (i==kPlot && sr2==0.0) continue;

    arcx=x[i];
    arcy=y[i];
    arcr=r[i];
    arcr2=r2[i];

    arcc=0;

    if (arcx+arcr<xmin || arcx-arcr>xmax ||
	arcy+arcr<ymin || arcy-arcr>ymax) continue;


      if (findcrossX1(arcx,arcr,xmin,&a0,&a1)>0) {
	arcs[arcc]=a1;
	arce[arcc]=a0+TWOPI;
	arcc++;
      }
      if (findcrossX1(arcx,arcr,xmax,&a0,&a1)>0) {
	arcs[arcc]=a0;
	arce[arcc]=a1;
	arcc++;
      }
      if (findcrossY1(arcy,arcr,ymin,&a0,&a1)>0) {
	arcs[arcc]=a1;
	arce[arcc]=a0+TWOPI;
	arcc++;
      }
      if (findcrossY1(arcy,arcr,ymax,&a0,&a1)>0) {
	arcs[arcc]=a0;
	arce[arcc]=a1;
	arcc++;
      }


    if (i!=kPlot && sr2>0.0) {
      int ret;
      ret=findcross(arcx,arcy,arcr,sx,sy,sr,&a0,&a1);
      if (ret==0 || ret==2) continue; /* Out of range */
      if (ret==2) continue;
      if (ret==1) {
	arcs[arcc]=a1;
	arce[arcc]=a0+TWOPI;
	arcc++;
      } 
    }

    /* for each atom */
    for(k=0;k<kPlot;k++){
      if (k==i) continue;
      if (findcross(arcx,arcy,arcr,x[k],y[k],r[k],&a0,&a1)>0) {
	iUse[iNum[i]]=1;
	iUsePl[i]=1;
	arcs[arcc]=a0;
	arce[arcc]=a1;
	arcc++;
      }
    }
    
    CheckArc(&arcc, arcs, arce);

    arcs[arcc]=TWOPI;
    a0=0.0;
    for(j=0;j<arcc+1;j++) {
      if (arcs[j]>a0) {
	if (nInt==0 && i<nPlot && 
	    AutoExpandRange2(nSec,iZNum,arcx,arcy,arcr,
			     a0,arcs[j])==0) return 0;
	if (i<nPlot) { /* Normal */
	  CavPlotPen(1);
	  CavPlotColor(iCol[iNum[i]]);
	  CavPlotArc(arcx,arcy,zsec,arcr,
		     a0,arcs[j],natm[iNum[i]]);
	} else if (i<kPlot) { /* ICave */
	  CavPlotPen(3);
	  CavPlotColor(iCol[iNum[i]]);
	  CavPlotArc(arcx,arcy,zsec,arcr,
		     a0,arcs[j],natm[iNum[i]]);
	} else {
	  CavPlotPen(1);
	  CavPlotColor(iCol[0]);
	  CavPlotArc(arcx,arcy,zsec,arcr,
		     a0,arcs[j],"SPHERE");
	}
      }
      a0=arce[j];
    }
  }

  if (nInt>0) {
    DrawSection(1,ymin,ymax,xmin,x,y,sx,sy,arcs,arce,"xmin");
    DrawSection(1,ymin,ymax,xmax,x,y,sx,sy,arcs,arce,"xmax");
    DrawSection(0,xmin,xmax,ymin,y,x,sy,sx,arcs,arce,"ymin");
    DrawSection(0,xmin,xmax,ymax,y,x,sy,sx,arcs,arce,"ymax");
  }
  CavPlotColor(0);
  return 1;
}

double PlVol(void) {
  double vy;
  double plvol=0;
  double amin,amax;
  int count;
  double st[MaxAtm],ed[MaxAtm];

  if (iVol==0) return 0.0;

  for(vy=ymin;vy<=ymax;vy+=divz) {
    amin=xmin;
    amax=xmax;
    plvol+=FindSection(&amin, &amax, vy, y, x, sy, sx, st, ed, &count);
  }
  return plvol*divz*divz;
}
 
/**************************************
      subroutine setpl
**************************************/
static void setpl(void) {
  int i,j,k;
  nPlot=0;
  sx=sxyz[kx];
  sy=sxyz[ky];
  sr=sr2=0.0;
  if (sradi!=0) {
    sr2=sradi*sradi-((sxyz[kz]-zsec)*(sxyz[kz]-zsec)); 
    if (sr2>=0) {
      sr=sqrt(sr2);
    } else {
      sr=0.0;
    }
  }
  for(i=0;i<nAtom;i++){
    double dist,dist2;
    if (xyz[i][kx]+w[i]<xmin || xyz[i][kx]-w[i]>xmax) continue;
    if (xyz[i][ky]+w[i]<ymin || xyz[i][ky]-w[i]>ymax) continue;
    if (bExclFlag[i]!=0) continue;
    dist=fabs(xyz[i][kz]-zsec);
    if (dist>w[i]) continue;
    dist2=dist*dist;
    x[nPlot]=xyz[i][kx];
    y[nPlot]=xyz[i][ky];
    name[nPlot]=natm[i];
    iNum[nPlot]=i;
    r2[nPlot]=w2[i]-dist2;
    r[nPlot]=sqrt(r2[nPlot]);
    nPlot++;
  }

  nEx=0;
  if (iCave!=0) {
    double dist,dist2;
    for(i=0;i<nExcl;i++){
      j=iExcl[i];
      dist=fabs(xyz[j][kz]-zsec);
      if (dist<=w[j]) {
	dist2=dist*dist;
	k=nPlot+nEx;
	nEx++;
	x[k]=xyz[j][kx];
	y[k]=xyz[j][ky];
	name[k]=natm[j];
	r2[k]=w2[j]-dist2;
	r[k]=sqrt(r2[k]);
      }
    }
  }
  kPlot=nPlot+nEx;
  x[kPlot]=sx;
  y[kPlot]=sy;
  r[kPlot]=sr;
  r2[kPlot]=sr2;

  fprintf(l_log,"ZSEC=%8.2f,NATOM=%5d\n",zsec,kPlot);
  if (iPrint!=0) {
    for(i=0;i<kPlot;i++){
      fprintf(l_log,"%-6s%10.6f%10.6f%10.6f\n",name[i],x[i],y[i],r[i]);
    }
  }
}

/******************************************
      Function IAutEx(NSEC,L,XP,YP)
******************************************/
static int AutoExpandRange(int nSec,int l,double xp,double yp) {
  if (cmin[kx]>xp) cmin[kx]=xp;
  if (cmax[kx]<xp) cmax[kx]=xp;
  if (cmin[ky]>yp) cmin[ky]=yp;
  if (cmax[ky]<yp) cmax[ky]=yp;
  if (cmin[kz]>zsec) cmin[kz]=zsec;
  if (cmax[kz]<zsec) cmax[kz]=zsec;

  if (xp<xmin){
    xmin-=fIncSt*divz;
    fprintf(l_log,"RECALCULATION XMIN =%6.2f\n",xmin);
    return 0;
  } else if (xp>xmax) {
    xmax+=fIncSt*divz;
    fprintf(l_log,"RECALCULATION XMAX =%6.2f\n",xmax);
    return 0;
  } else if (yp<ymin) {
    ymin-=fIncSt*divz;
    fprintf(l_log,"RECALCULATION YMIN =%6.2f\n",ymin);
    return 0;
  } else if (yp>ymax) {
    ymax+=fIncSt*divz;
    fprintf(l_log,"RECALCULATION YMAX =%6.2f\n",ymax);
    return 0;
  }
  return 1;
}

static int AutoExpandRange2(int nSec,int l,
			    double x,double y,double r, double st, double ed) {
  if (l==0) {
    zmin-=fIncSt*divz;
    fprintf(l_log,"RECALCULATION ZMIN =%6.2f\n",zmin);
    return 0;
  }
  if (l>=nSec) {
    zmax=zmax+fIncSt*divz;
    fprintf(l_log,"RECALCULATION ZMAX =%6.2f\n",zmax);
    return 0;
  }

  if (0==AutoExpandRange(nSec,l,x+r*cos(st),y+r*sin(st))) return 0;
  if (0==AutoExpandRange(nSec,l,x+r*cos(ed),y+r*sin(ed))) return 0;
  if (st<0.5*PI && ed>0.5*PI &&
      0==AutoExpandRange(nSec,l,x,y+r)) return 0;
  if (st<1.5*PI && ed>1.5*PI &&
      0==AutoExpandRange(nSec,l,x,y-r)) return 0;
  if (st<PI && ed>PI &&
      0==AutoExpandRange(nSec,l,x-r,y)) return 0;
  return 1;
}


/******************************************
      subroutine CheckRange
c
c     Automatic Range Expand Check
c
******************************************/
static void CheckRange(void) {
  if (nInt!=0) return;
  if (zmin<-fMaxAr) {
    error("Auto Range Over Flow / ZMIN !!");
  } else if (zmax>fMaxAr) {
    error("Auto Range Over Flow / ZMAX !!");
  } else if (xmin<-fMaxAr) {
    error("Auto Range Over Flow / XMIN !!");
  } else if (xmax>fMaxAr) {
    error("Auto Range Over Flow / XMAX !!");
  } else if (ymin<-fMaxAr) {
    error("Auto Range Over Flow / YMIN !!");
  } else if (ymax>fMaxAr) {
    error("Auto Range Over Flow / YMAX !!");
  }
}
