/* -*- C -*- */
/*
c
c     Cavity Common Parameters / Common Variables
c


c     ----------------------
c       Common Parameters
c     ----------------------

ccc   input / output device 
c     data_fp = cav.dat
c     cnt_fp = cav.cnt
c     log_fp = cav.log
c     ps_fp = cav.ps
c     res_fp = cav.res
*/

#include <stdio.h>
#include <math.h>

extern FILE *l_data,*l_log,*l_meta;

/*   Maximum Number Of Atoms in the DataFile */
#define MaxAtm 20000

/*   Maximum Number Of Atoms in one of the section of Cavity */
#define MaxAPl 1000

/*   Maximum Number Of Bonds */
#define MaxBnd 3000

/*   Maximum Number Of Bonds shown in the Figure */
#define MaxPlt 3000

/*   Maximum Number Of Atoms in the Cavity */
#define MaxExc 3000

/*   Maximum Number Of Elements */
#define MaxVDW 10

/*   Maximum Number Of Pens (Don't Edit) */
#define MaxPen 4

/*   Automatic Range Extention Limit (from gravity center, in angstrome) */
#define fMaxAr 20.0

/*   Automatic Range Extention  Step Width (in angstrome) */
#define fIncSt 10.0

/*   Initial Calculation Range Distance from atoms in cavity (in angstrome) */
#define cAdd 1.5



/* Global Variables (in global.c) */

extern char title[80];
extern int iView,nInt,iVol,iPlot,iPrint,iCave,nPlt,nBond,nExcl;
extern double scale,radi;
extern int *iPen;
extern int *iPlt,*iPs,*iPe,*iExcl;
extern double *Vdw;
extern double xmin,xmax,ymin,ymax,zmin,zmax,divz;
extern double *x,*y,*r,*r2;
extern double sx,sy,sr,sr2;
extern char **name;
extern int *iNum,nPlot,kPlot,nAtom,nEx;
extern char (*natm)[10];
extern int *iTbl,*iCol;
extern double (*xyz)[3],*w,*w2;
extern double cmin[3],cmax[3],vmt[3][3],rvmt[3][3],org[3];
extern int kx,ky,kz;
extern char lbuf[1024];
extern int iErr;
extern double sxyz[3],sradi;
extern int *iUse,*iUsePl;
extern double vol,plvol;


extern int *bExclFlag;

/* cavity.c */
extern void error(char *str);
/* readdt.c */
extern int ReadDataFile(const char *fileName);
extern int ReadControlFile(const char *fileName);
extern int CleanStructure(void);
/* plot.c */
extern int CavPlotArc(double fx,double fy,double fz,double fr,
		   double fst,double fed,char *name1);
extern int CavOpenPlotter(const char *fileName);

extern int CavPlotLine(double fx,double fy,double fz,
		    double fx2,double fy2,double fz2,
		    char *name1,char *name2);
extern int CavPlotPen(int penNumber);
extern int CavPlotMatrix(void);

extern int CavClosePlotter(void);
extern int CavPlotTitle(char *str);
extern int CavPlotComment(char *mess);
extern int CavPlotColor(int colorNumber);
extern int CavPlotFlush(void);

extern int CavPlotMeshDirection(int num);
extern int CavPlotMeshBegin(int num);
extern int CavPlotMeshEnd(void);


extern int CavInit(const char *filename);
extern void cavin2(void);
extern int CavTrailer(void);

extern int GlobalInit(void);

#define TWOPI (atan(1.0)*8.0)
#define PI (atan(1.0)*4.0)
#define degree(x) ((double)x*180.0/PI)
#define radian(x) ((double)x*PI/180.0)

