/* -*- C -*- */
/* $Id: plot.c 195 2008-09-04 13:07:38Z tnemoto $ */

#ifdef HAVE_CONFIG_H
#include "../config.h"
#endif

#ifdef HAVE_TIME_H
#include <time.h>
#endif

#include "cavity.h"

static int num_pen;
static double bx0=10000.0,by0=10000.0,bx1=-10000.0,by1=-10000.0;
static long boundingBoxPointer;

static int PovOpen(void);
static int PovClose(void);
static int PovArc(double fx,double fy,double fz, double fr, double fst,double fed, const char *name1);

FILE *l_meta, *pfp;

#define BoundingBoxString "%%%%BoundingBox: %6d %6d %6d %6d\n"

int CavPlotArc(double fx,double fy,double fz,double fr,
	    double fst,double fed,char *name1) {
  if (l_meta == NULL) return 0;
  fprintf(l_meta,"ca  %8.3f %8.3f %8.3f %8.3f %8.3f a %% %8.3f    %-6s\n",
	  fx,fy,fr,degree(fst),degree(fed),fz,name1);
  PovArc(fx,fy,fz, fr, degree(fst), degree(fed), name1);
  if (bx1<fx+fr*cos(fst)) {
    bx1=fx+fr*cos(fst);
  }
  if (bx1<fx+fr*cos(fed)) {
    bx1=fx+fr*cos(fed);
  }
  if (fst<=PI && fed>=PI && bx0>fx-fr) {
    bx0=fx-fr;
  } 
  if (fx+fr*cos(fst)<bx0) {
    bx0=fx+fr*cos(fst);
  }
  if (fx+fr*cos(fed)<bx0) {
    bx0=fx+fr*cos(fed);
  }
  if (fst<=0.5*PI && fed>=0.5*PI && by1<fy+fr) {
    by1=fy+fr;
  } 
  if (by1<fy+fr*sin(fst)) {
    by1=fy+fr*sin(fst);
  }
  if (by1<fy+fr*sin(fed)) {
    by1=fy+fr*sin(fed);
  }
  if (fst<=1.5*PI && fed>=1.5*PI && by0>fy-fr) {
    by0=fy-fr;
  } 
  if (fy+fr*sin(fst)<by0) {
    by0=fy+fr*sin(fst);
  }
  if (fy+fr*sin(fed)<by0) {
    by0=fy+fr*sin(fed);
  }
  return 1;
}

int CavOpenPlotter(const char *fileName) {
#ifdef CREATION_TIME
  time_t tt;
  tt=time(NULL);
#endif  

  l_meta=fopen(fileName,"w+");
  if (l_meta==NULL) error("Can't Open Output File!");

  num_pen=-1;
  fprintf(l_meta,"%%!PS-Adobe-3.0 EPSF-3.0\n");
  fprintf(l_meta,"%%%%Creater: PS-META-FILE (Cavity %s)\n",VERSION);
#ifdef CREATION_TIME
  fprintf(l_meta,"%s %s\n","%%CreationDate: ",asctime(localtime(&tt)));
#endif
  /*
  c      fprintf(l_meta,"%s\n","%%DocumentMedia: ");
  c      fprintf(l_meta,"%s\n","%%DocumentPrinterRequired: ");
  c      fprintf(l_meta,"%s\n","%%For:");
  c      fprintf(l_meta,"%s\n","%%PageMedia:");
  c      fprintf(l_meta,"%s\n","%%PageRequirements:");
  */
  boundingBoxPointer=ftell(l_meta);
  fprintf(l_meta,BoundingBoxString,0,0,0,0);
  fprintf(l_meta,"%s\n","%%Pages: 1");
  fprintf(l_meta,"%s\n","%%EndComment");
  fprintf(l_meta,"%s\n","%%EndProlog");
  fprintf(l_meta,"save\n");
  fprintf(l_meta,"/color { false } def\n");
  fprintf(l_meta,"/color2 { false } def\n");
  fprintf(l_meta,"/penwidth {[0.001 0.001 0.005 0.050 0.001]} def\n");
  fprintf(l_meta,"/pengray {[0.000 0.000 0.000 0.000 0.700]} def\n");
  fprintf(l_meta,"/penred {[0.000 0.000 0.000 1.000 0.700]} def\n");
  fprintf(l_meta,"/pengreen {[0.000 0.000 0.000 0.000 0.700]} def\n");
  fprintf(l_meta,"/penblue {[0.000 1.000 0.000 0.000 0.700]} def\n");

  fprintf(l_meta,"/colorblue {[0 1 0 1 0 1 0 1]} def\n");
  fprintf(l_meta,"/colorred {[0 0 1 1 0 0 1 1]} def\n");
  fprintf(l_meta,"/colorgreen {[0 0 0 0 1 1 1 1]} def\n");

  fprintf(l_meta,"/m { moveto } def\n");
  fprintf(l_meta,"/s { stroke } def\n");
  fprintf(l_meta,"/l { /liney exch def /linex exch def\n");
  fprintf(l_meta,"    m linex liney lineto s} def\n");
  fprintf(l_meta,"/a { /arced exch def /arcst exch def\n");
  fprintf(l_meta,"/arcr exch def /arcy exch def\n");
  fprintf(l_meta,"/arcx exch def arcst cos arcr mul arcx add\n");
  fprintf(l_meta,"arcst sin arcr mul arcy add m \n");
  fprintf(l_meta,"arcx arcy arcr arcst arced arc s} def\n");
  fprintf(l_meta,"/C { /col exch def  color2 {\n");
  fprintf(l_meta,"colorred col get\n");
  fprintf(l_meta,"colorgreen col get\n");
  fprintf(l_meta,"colorblue col get\n");
  fprintf(l_meta,"setrgbcolor\n");
  fprintf(l_meta,"} if } def\n");
  fprintf(l_meta,"/p { dup penwidth exch get setlinewidth\n");
  fprintf(l_meta,"     color { /col exch def\n");
  fprintf(l_meta,"        penred col get pengreen col get\n");
  fprintf(l_meta,"        penblue col get setrgbcolor \n");
  fprintf(l_meta,"     } {pengray exch get setgray }\n");
  fprintf(l_meta,"     ifelse  } def\n");
  fprintf(l_meta,"/rm { rmoveto } def\n");
  fprintf(l_meta,"/rl { rlineto } def\n");
  fprintf(l_meta,"/tt {pop} def\n");
  fprintf(l_meta,"/ar { } def\n");
  fprintf(l_meta,"/cm {} def\n");
  fprintf(l_meta,"/cl {} def\n");
  fprintf(l_meta,"/crm {} def\n");
  fprintf(l_meta,"/crl {} def\n");
  fprintf(l_meta,"/cp {} def\n");
  fprintf(l_meta,"/ct {} def\n");
  fprintf(l_meta,"/ca {} def\n");
  fprintf(l_meta,"/cc {} def\n");
  fprintf(l_meta,"/cC {} def\n");
  fprintf(l_meta,"%% 50 50 m 500 0 rl 0 500 rl -500 0 rl\n");
  fprintf(l_meta,"%% closepath clip newpath\n");
  fprintf(l_meta,"%10.4f%10.4f%s\n",300.0,300.0," translate");
  fprintf(l_meta,"%10.4f%10.4f%s\n",50.0,50.0," scale");
  fprintf(l_meta,"%% View Control\n");
  fprintf(l_meta,"%10.4f%10.4f%s\n",scale,scale," scale");
  fprintf(l_meta,"%10.4f%10.4f%s\n",0.0,0.0," translate");
  fprintf(l_meta,"%s\n","%# StartData");
  fprintf(l_meta,"%%# ORIGIN=%10.6f%10.6f%10.6f\n",org[0],org[1],org[2]);
  fflush(l_meta);
  PovOpen();
  return 1;
}

int CavPlotLine(double fx,double fy,double fz,double fx2,double fy2,double fz2,
	    char *name1,char *name2){
  if (l_meta == NULL) return 0;
  fprintf(l_meta,"cl  %8.3f%8.3f%8.3f%8.3f l %% %8.3f%8.3f %-6s %-6s\n",
	  fx,fy,fx2,fy2,fz,fz2,name1,name2);
  if (fx<bx0) bx0=fx;
  if (fx2<bx0) bx0=fx2;
  if (fx>bx1) bx1=fx;
  if (fx2>bx1) bx1=fx2;
  if (fy<by0) by0=fy;
  if (fy2<by0) by0=fy2;
  if (fy>by1) by1=fy;
  if (fy2>by1) by1=fy2;
  return 1;
}

int CavPlotPen(int i){
  if (l_meta == NULL) return 0;
  if (i!=num_pen) {
    num_pen=i;
    fprintf(l_meta,"cp  %7d p\n",iPen[i-1]);
  }
  return 1;
}

int CavClosePlotter(void) {
  if (l_meta==NULL) return 0;
  fprintf(l_meta,"%%# EndData\n");
  fprintf(l_meta,"%%%%Trailer\n");
  fprintf(l_meta,"restore\n");
  fprintf(l_meta,"showpage\n");
  fprintf(l_meta,"%%%%EndTrailer\n");
  fprintf(l_meta,"%%%%EOF\n");
  fseek(l_meta,boundingBoxPointer,SEEK_SET);
  fprintf(l_meta,BoundingBoxString,
	  (int)(bx0*50*scale+300-1),
	  (int)(by0*50*scale+300-1),
	  (int)(bx1*50*scale+300+1),
	  (int)(by1*50*scale+300+1));
  fclose(l_meta);
  l_meta=NULL;
  PovClose();
  return 1;
}

int CavPlotTitle(char *title) {
  if (l_meta == NULL) return 0;
  fprintf(l_meta,"ct (%-40s) tt\n",title);
  return 1;
}


int CavPlotComment(char *mess) {
  if (l_meta == NULL) return 0;
  fprintf(l_meta,"%%# mess %-40s\n",mess);
  fflush(l_meta);
  return 1;
}

int plotcm(char *mess) {
  if (l_meta == NULL) return 0;
  fprintf(l_meta,"%%# %s\n",mess);
  return 1;
}

int CavPlotFlush(void) {
  if (l_meta == NULL) return 0;
  fflush(l_meta);
  return 1;
}

int CavPlotMeshDirection(int num) {
  if (l_meta == NULL) return 0;
  fprintf(l_meta,"cc /mesh {%d} def\n",num);
  return 1;
}

int CavPlotMeshBegin(int num) {
  if (l_meta == NULL) return 0;
  fprintf(l_meta,"cc save mesh %d ne { \n",num);
  fprintf(l_meta,
	  "cc -100 -100 m 0 0 rl 0 0 rl 0 0 rl 0 0 rl clip newpath } if\n");
  return 1;
}

int CavPlotMeshEnd(void) {
  if (l_meta == NULL) return 0;
  fprintf(l_meta,"cc restore\n");
  return 1;
}

int CavPlotMatrix(void) {
  int i;
  if (l_meta == NULL) return 0;
  for(i=0;i<3;i++){
    fprintf(l_meta,"%%# RVMT%1d =%10.6f%10.6f%10.6f\n",
	    i+1,rvmt[i][kx],rvmt[i][ky],rvmt[i][kz]);
  }
  return 1;
}

int CavPlotColor(int num) {
  if (l_meta == NULL) return 0;
  fprintf(l_meta,"cC %2d C\n",num);
  return 1;
}


/* ============================================================== */
/********************************************************
      subroutines for povray file
********************************************************/
static const char *povradii(const char *name){
  if (name[0]=='C' && !isalpha(name[1])) return "radC";
  if (name[0]=='H' && !isalpha(name[1])) return "radH";
  if (name[0]=='N' && !isalpha(name[1])) return "radN";
  if (name[0]=='O' && !isalpha(name[1])) return "radO";
  if (name[0]=='C' && name[1]=='L') return "radCL";
  return "radX";
}

static const char *povcolor(const char *name) {
  if (name[0]=='C' && !isalpha(name[1])) return "catC";
  if (name[0]=='H' && !isalpha(name[1])) return "catH";
  if (name[0]=='N' && !isalpha(name[1])) return "catN";
  if (name[0]=='O' && !isalpha(name[1])) return "catO";
  if (name[0]=='C' && name[1]=='L') return "catCL";
  return "catX";
}

static int PovOpen(void) {

  pfp=fopen("cav.pov","w");
  if (pfp==NULL) {
    error("Can't open cav.pov\n");
    return 0;
  }

  fprintf(pfp,
"#include \"textures.inc\"\n"
"#include \"colors.inc\"\n"
"#include \"metals.inc\"\n"
"\n"
"#declare SCALE = 1.0;\n"
"#declare TRANSLATE = <0,0,0>;\n"
"#declare DISTANCE = 30;\n"
"#declare STEREO = 0;\n"
"#declare STEREOSPLIT = 5;\n"
"#declare PERSPECTIVE = 0;\n"
"#declare ASPECTRATIO = image_width/image_height;\n"
"// #declare SKIN = 1.2;\n"
	  "\n\n\n");
  fputs(
"  light_source {< 25, 25, 50> color rgb <1,1,1>  adaptive 1 }\n"
"  light_source {< 0, 0, 100> color rgb <1,1,1>  adaptive 1 shadowless}\n"
"  // light_source {< 0, 0, 100> color rgb <1,1,1>  adaptive 1 shadowless}\n"
"  // light_source {< 0, 100, 0> color rgb <1,1,1>  adaptive 1 shadowless}\n"
"  // light_source {< 0, -100, 0> color rgb <1,1,1>  adaptive 1 shadowless}\n"
,pfp);
  fputs(
"  // light_source {<100, 0, 0> color rgb <1,1,1>  adaptive 1 shadowless}\n"
"  // light_source {<-100, 0, 0> color rgb <1,1,1>  adaptive 1 shadowless}\n"
"#if (PERSPECTIVE)\n"
"  camera { perspective right -x*ASPECTRATIO up y location <0,0,DISTANCE> look_at <0,0,0> }\n"
"#else\n"
"  camera { orthographic right -x*DISTANCE*ASPECTRATIO up y*DISTANCE location <0,0,DISTANCE> look_at <0,0,0> }\n"
"#end\n"
"  background { color <0.5,0.5,0.5> }\n"
"\n",pfp);

  fprintf(pfp,
"// Cavity Color\n"
"#declare CAVCOLOR = color rgbt <0.8,1.0,1.0,0.5>; // transparent light blue\n" 
"#declare BOXCOLOR = color rgbt <0.5,0.5,0,0.5>; // transparent yellow \n"
"// Element Color\n"
"#declare catC = color rgb <0.5,0.5,0.5>;\n"
"#declare catH = color rgb <0.0,1.0,1.0>;\n"
"#declare catN = color rgb <0.0,0.0,0.7>;\n"
"#declare catO = color rgb <1.0,0.1,0.1>;\n"
"#declare catCL = color rgb <0.0,0.7,0.0>;\n"
"#declare catX = color rgb <0.3,0.3,0.3>;\n"
"// Element Radius\n"
"#declare radC = 0.4;\n"
"#declare radH = 0.2;\n"
"#declare radN = 0.4;\n"
"#declare radO = 0.4;\n"
"#declare radCL = 0.5;\n"
"#declare radX = 0.6;\n"
"// Bond Radius\n"
"#declare bondw = 0.1;\n"
"// Skin Thickness\n"
"#declare T0 = 0.001;\n"
	  );

  fputs(
"#macro ARC (CENT,R0,R1,ABEGIN,AEND)\n"
"  merge{\n"
"  #if (AEND-ABEGIN>180)\n"
"    sphere { <-R0,0,0> R1 }\n"
"    intersection{\n"
"      torus {R0, R1 rotate<90,0,0> }\n"
"      box { <-R0-R1,0,-R1> <R0+R1,-R0-R1,R1> }\n"
"        #local ABEGIN=ABEGIN+180;\n"
"    }\n"
"  #end\n"
"    merge{\n"
"      intersection {\n"
"        intersection {\n"
"          torus { R0, R1 rotate <90,0,0> }\n"
"          box { <-R0-R1,0,-R1> <R0+R1,R0+R1,R1> }\n"
"          rotate <0,0,-AEND+ABEGIN>\n"
"        }\n"
"        box { <-R0-R1,0,-R1> <R0+R1,-R0-R1,R1> }\n"
"      }\n"
"      sphere { <R0,0,0> R1 }\n"
"      rotate<0,0,-ABEGIN+AEND>\n"
"    }\n"
"    sphere { <R0,0,0> R1 }\n"
"    rotate<0,0,ABEGIN>\n"
"    translate CENT\n"   
"  }\n"
"#end\n"
"\n\n",pfp);


  fprintf(pfp,
"#ifdef (SKIN)\n"
" #if (SKIN>0)\n"
"  #declare THICKSKIN = merge {\n"
	  );
  return 1;
}

static int PovArc(double fx,double fy, double fz, double fr, 
		  double fst,double fed, const char *name1){
  if (pfp==NULL) return 0;
  fprintf(pfp,"   ARC(<%.3f,%.3f,%.3f>,%.3f,SKIN,%.3f,%.3f) // %s\n",
	  fx,fy,fz,fr,fst,fed,name1);
  return 1;
};
static int PovClose() {
  int gsc=0,i,j;
  double cavr[3],aavr[3];

  if (pfp==NULL) return 0;

  fprintf(pfp,
	  "  };\n"
	  " #end\n"
	  "#end\n");

  for(i=0; i<3; i++) {
    aavr[i]=0.0;
  }
  for(i=0; i<3; i++) {
    cavr[i]=(cmin[i]+cmax[i])*0.5;
  }
  fprintf(pfp,
	  "#declare RANGEBOX = box { <%.3f,%.3f,%.3f>,<%.3f,%.3f,%.3f> } \n",
	  cmin[0], cmin[1], cmin[2], cmax[0], cmax[1], cmax[2]);
  fprintf(pfp,
	  "#declare RANGEBOX2 = difference {\n"
	  "  box { <%.3f-T0,%.3f-T0,%.3f-T0>,<%.3f+T0,%.3f+T0,%.3f+T0> }"
	  "  box { <%.3f+T0,%.3f+T0,%.3f+T0>,<%.3f-T0,%.3f-T0,%.3f-T0> }"
	  "}\n",
	  cmin[0], cmin[1], cmin[2], 
	  cmax[0], cmax[1], cmax[2],
	  cmin[0], cmin[1], cmin[2], 
	  cmax[0], cmax[1], cmax[2]);
  fprintf(pfp,
	  "#declare RANGEBOX_Center =<%f,%f,%f>;\n",cavr[0],cavr[1],cavr[2]);

  if (sradi>0) {
    fprintf(pfp,
	  "#declare SPHERE = sphere { <%.3f,%.3f,%.3f>, %.3f }\n",
	  sxyz[0],sxyz[1],sxyz[2],sradi);
    fprintf(pfp,
	    "#declare SPHERE2 = difference {\n"
	    "  sphere { <%.3f,%.3f,%.3f>, %.3f+T0 }\n"
	    "  sphere { <%.3f,%.3f,%.3f>, %.3f-T0 }\n"
	    "}\n",
	    sxyz[0],sxyz[1],sxyz[2],sradi,
	    sxyz[0],sxyz[1],sxyz[2],sradi);
    fprintf(pfp,
	    "#declare BOX = \n"
	    "  intersection { object {RANGEBOX} object {SPHERE} };\n"
	    "#declare BOX2 = \n"
	    "  merge { object {RANGEBOX2} object {SPHERE2} };\n"
	    );
  } else {
    fprintf(pfp,
	    "#declare BOX = object {RANGEBOX};\n"
	    "#declare BOX2 = object {RANGEBOX2};\n"
	    );
  }
  fprintf(pfp,
	  "#declare CAV =\n"
	  "  merge {\n");
  for(i=0;i<nAtom;i++){
    if (iUse[i]){
      fprintf(pfp, "   sphere { <%.3f,%.3f,%.3f>, %.3f } // %s_%d\n",
	      xyz[i][0],xyz[i][1],xyz[i][2],w[i],natm[i],i+1);
    }
  }

  fprintf(pfp,
	  "}\n\n\n");
  fprintf(pfp,"// end of CAV\n");

  fprintf(pfp,
"#declare CAV2 =\n"
"  merge {\n");
  for(i=0;i<nAtom;i++){
    if (iUse[i]){
      fprintf(pfp, 
     "   sphere { <%.3f,%.3f,%.3f>, %.3f +T0 pigment {%s}} // %s_%d\n",
	      xyz[i][0],xyz[i][1],xyz[i][2],w[i],povcolor(natm[i]),natm[i],i+1);
    }
  }
  fprintf(pfp,
	  "}\n"
	  "// end of CAV2\n");


  fprintf(pfp, "#declare ATOMS =\n"
	  " union {\n");

  for(i=0;i<nPlt;i++){
    int k;
    k=iPlt[i];
    fprintf(pfp, "   sphere { <%.3f,%.3f,%.3f>, %s pigment {%s}} // %s_%d\n",
	    xyz[k][0],xyz[k][1],xyz[k][2],
	    povradii(natm[k]),povcolor(natm[k]),natm[k],k+1);
    for(j=0;j<3;j++) {
      aavr[j]+=xyz[k][j];
    }
    gsc++;
  }

  fprintf(pfp, " } // end of Atoms\n");


  fprintf(pfp, "#declare CAVATOMS =\n"
	  " union {\n");

  for(i=0;i<nExcl;i++){
    int k;
    k=iExcl[i];
    fprintf(pfp, "   sphere { <%.3f,%.3f,%.3f>, %s pigment {%s}} // %s_%d\n",
	    xyz[k][0],xyz[k][1],xyz[k][2],
	    povradii(natm[k]),povcolor(natm[k]),natm[k],k+1);
  }

  fprintf(pfp, " } // end of CAVATOMS\n");



  fprintf(pfp, "#declare MODEL =\n"
	  " union {\n");

  for(i=0;i<nBond;i++){
    double *sxyz,*exyz;
    sxyz=xyz[iPs[i]];
    exyz=xyz[iPe[i]];
    for(j=0;j<3;j++) cavr[j]=(sxyz[j]+exyz[j])*0.5;
    fprintf(pfp, 
    "  cylinder { <%.3f,%.3f,%.3f>,<%.3f,%.3f,%.3f>,bondw pigment {%s} } // %s_%d -- \n",
	    cavr[0],cavr[1],cavr[2],
	    sxyz[0],sxyz[1],sxyz[2],
	    povcolor(natm[iPs[i]]),natm[iPs[i]],iPs[i]+1);
    fprintf(pfp, 
    "  cylinder { <%.3f,%.3f,%.3f>,<%.3f,%.3f,%.3f>,bondw pigment {%s} } // -- %s_%d \n",
	    cavr[0],cavr[1],cavr[2],
	    exyz[0],exyz[1],exyz[2],
	    povcolor(natm[iPe[i]]),natm[iPe[i]],iPe[i]+1);
  }

  fprintf(pfp, " } // end of Model\n");

  fprintf(pfp,
"#declare CAV0 = \n"
"	  difference {\n"
"	    object {BOX}\n"
"	    object {CAV}\n"
"	  }\n"
"#ifdef (SKIN)\n"
"  #ifdef (THICKSKIN)\n"
"    #declare CAVITY= \n"
"	merge { object{CAV0} object{THICKSKIN} }\n"
"  #else\n"
"    #declare CAVITY = \n"
"       intersection {\n"
"	  union { object{CAV2} object{BOX2} }\n"
"	  object {CAV0 pigment { color rgbt <1,0,0,1>}}\n"
"	}\n"
"  #end\n"
"#else\n"
"#declare CAVITY = object {CAV0};\n"
"#end\n"
	  );

  fprintf(pfp,"// =================================\n");
  fprintf(pfp,"//           View Control\n");
  fprintf(pfp,"// =================================\n");

  fprintf(pfp,"#declare CENTER=<%.3f,%.3f,%.3f>;\n",
	  aavr[0]/gsc,aavr[1]/gsc,aavr[2]/gsc);

  fprintf(pfp, 
	  "#declare FULL=\n"
	  " union {\n"
	  "  object {MODEL} \n"
	  "  object {CAVITY pigment { CAVCOLOR } }\n" 
	  "  // object {ATOMS} \n"
	  "  // object {CAVATOMS} \n"
	  "  // object {BOX pigment { BOXCOLOR } }\n"
	  );
  fprintf(pfp, 
	  "  translate -CENTER\n"
	  "  rotate <0,clock*360,0>\n"
	  "  scale SCALE\n"
	  "  translate TRANSLATE\n"
	  " }\n");
  fprintf(pfp,
	  "#if (STEREO)\n"
	  " #if (PERSPECTIVE)\n"
	  "  #declare srot=atan2(STEREOSPLIT,DISTANCE)/3.1415*180;"
	  " #else\n"
	  "  #declare srot=0;"
	  " #end\n"
	  "  object {FULL rotate <0,srot+5,0> translate <-STEREOSPLIT,0,0> }\n"
	  "  object {FULL rotate <0,-srot-5,0> translate <STEREOSPLIT,0,0> }\n"
	  "#else\n"
	  "  object {FULL}\n"
	  "#end\n"
	  );


  fclose(pfp);



  pfp=fopen("cav.ini","r");
  if (pfp!=NULL) {
    fclose(pfp);
  } else {
    pfp=fopen("cav.ini","w");
    if (pfp!=NULL) {
      fprintf(pfp,
	      "Width=320\n"
	      "Height=240\n"
	      "Antialias=Off\n"
	      "Antialias_Threshold=0.3\n"
	      "Antialias_Depth=3\n"
	      "Input_File_Name=cav.pov\n"
	      "Initial_Frame=1\n"
	      "Final_Frame=50\n"
	      "Initial_Clock=0\n"
	      "Final_Clock=1\n"
	      "Pause_when_Done=off\n"
	      "Cyclic_Animation=off\n");
      fclose(pfp);
    }
  }
  return 1;
}
