/* -*- C -*- */

#include "cavity.h"
#include <string.h>
#include <stdlib.h>

static int iGetCl(char *cho);

static void chkcom(FILE *fp) {
  int i;
  for(i=0;i<80;i++)lbuf[i]=0;
  do {
    fgets(lbuf,80,fp);
  } while (lbuf[0]=='#');
}

int ReadDataFile(const char *fileName) {
  /*     Read Atom Coordinate */
  double ftmp[3];
  FILE *l_data;
  int i,j;
  l_data=fopen(fileName,"r");
  if (l_data==NULL) return 0;
  nAtom=0;
  for(i=0;i<3;i++){
    for(j=0;j<3;j++){
      vmt[i][j]=rvmt[i][j]=0;
    }
    vmt[i][i]=rvmt[i][i]=1;
    org[i]=0;
    sxyz[i]=0;
  }
  
  sradi=0;
  for(i=0;i<MaxAtm;i++){
    char lbuf[80];
    if (fgets(lbuf,80,l_data)==NULL) break;
    if (lbuf[0]=='#') {
      i--;
      continue;
    }
    memcpy(natm[i],lbuf,6);
    natm[i][6]=0;
    sscanf(lbuf+6,"%4d",iTbl+i);
    sscanf(lbuf+10,"%10lf%10lf%10lf",xyz[i],xyz[i]+1,xyz[i]+2);
    if (strcmp(natm[i],"ORIGIN")==0) {
      for(j=0;j<3;j++){
	org[j]=xyz[i][j];
      }
      i--;
      continue;
    } else if (strncmp(natm[i],"VMT",3)==0) {
      j=natm[i][3]-'0';
      sscanf(lbuf+10,"%10lf%10lf%10lf",vmt[j],vmt[j]+1,vmt[j]+2);
      i--;
      continue;
    } else if (strncmp(natm[i],"RVMT",4)==0) {
      j=natm[i][4]-'0';
      sscanf(lbuf+10,"%10lf%10lf%10lf",rvmt[j],rvmt[j]+1,rvmt[j]+2);
      i--;
      continue;
    } else if (strcmp(natm[i],"SPHERE")==0) {
      sscanf(lbuf+10,"%10lf%10lf%10lf%10lf",sxyz,sxyz+1,sxyz+2,&sradi);
      for(j=0;j<3;j++){
	ftmp[j]=sxyz[j]-org[j];
      }
      for(j=0;j<3;j++){
	sxyz[j]=vmt[j][0]*ftmp[0]+vmt[j][1]*ftmp[1]+vmt[j][2]*ftmp[2];
      }
      i--;
      continue;
    }
    iCol[i]=iGetCl(natm[i]);
    for(j=0;j<3;j++){
      ftmp[j]=xyz[i][j]-org[j];
    }
    for(j=0;j<3;j++){
      xyz[i][j]=vmt[j][0]*ftmp[0]+vmt[j][1]*ftmp[1]+vmt[j][2]*ftmp[2];
    }
    nAtom++;
  }
  fclose(l_data);
  if (i==MaxAtm) error("Too Many Atoms !!");
  return 1;
}


static int iGetCl(char *cho) {
  char ch0,ch1;
  ch0=cho[0];
  ch1=cho[1];
  if (ch1>='0' && ch1<='9') ch1=' ';
  if (ch0=='C' && ch1==' ') return 0;
  if (ch0=='H' && ch1==' ') return 5;
  if (ch0=='N' && ch1==' ') return 4;
  if (ch0=='O' && ch1==' ') return 2;
  if (ch0=='C' && ch1=='O') return 3;
  if (ch0=='P' && ch1==' ') return 3;
  return 0;
}

/**********************************************************************
c
c     Read Control File
c
 **********************************************************************/
								       
int ReadControlFile(const char *fileName) {
  int i,j;
  FILE *fp;
  fp=fopen(fileName,"r");
  if (fp==NULL) return 0;
  chkcom(fp);
  strncpy(title,lbuf,80);
  title[79]=0;
  for(i=0;i<80;i++){
    if (title[i]==10 || title[i]==13) title[i]=0;
  }
  chkcom(fp);
  sscanf(lbuf,"%7d%7d%7d%7d%7d%7d%7d%7d%7d",&iView,&nInt,&iVol,&iPlot,&iPrint,
	 &iCave,&nPlt,&nBond,&nExcl);
  chkcom(fp);
  sscanf(lbuf,"%7lf%7lf%7d%7d%7d%7d",
	 &scale,&radi,iPen,iPen+1,iPen+2,iPen+3);
  if (scale==0) scale=1.0;
  for(i=0;i<MaxPen;i++){
    if (iPen[i]==0) iPen[i]=i+1;
  }
  if (nPlt>0) {
    for(j=0;j<nPlt;j+=10) {
      chkcom(fp);
      for(i=0;i<10;i++){
	int d;
	sscanf(lbuf+i*7,"%7d",&d);
	iPlt[j+i]=d-1;
      }
    }
  }
  if (nBond>0) {
    for(j=0;j<nBond;j+=5) {
      chkcom(fp);
      for(i=0;i<5;i++){
	int d1,d2;
	if (i+j>=nBond) break;
	sscanf(lbuf+i*14,"%7d%7d",&d1,&d2);
	iPs[i+j]=d1-1;
	iPe[i+j]=d2-1;
      }
    }
  }

  for(j=0;j<nExcl;j+=10) {
    chkcom(fp);
    for(i=0;i<10;i++){
      int d;
      if (i+j>=nExcl) break;
      sscanf(lbuf+i*7,"%7d",&d);
      iExcl[i+j]=d-1;
    }
  }

  for(j=0;j<MaxVDW;j+=10) {
    chkcom(fp);
    for(i=0;i<10;i++){
      sscanf(lbuf+i*7,"%7lf",Vdw+i+j);
    }
  }

  chkcom(fp);
  sscanf(lbuf,"%7lf%7lf%7lf%7lf%7lf%7lf%7lf",
	 cmin,cmax,cmin+1,cmax+1,cmin+2,cmax+2,&divz);
  fclose(fp);

  return 1;
}


int CleanStructure(void) {
  int i,j,k;
  int *atomMap;
  double x0,x1,y0,y1,z0,z1;
  atomMap=(int *)malloc(sizeof(int)*nAtom);
  if (atomMap==NULL) return 0;
  for(i=0;i<nAtom;i++){
    atomMap[i]=i;
    bExclFlag[i]=0;
  }
  for(i=0, k=0;i<nAtom;i++){
    x0=xyz[i][0];
    y0=xyz[i][1];
    z0=xyz[i][2];
    for(j=0;j<k;j++){
      x1=xyz[j][0];
      y1=xyz[j][1];
      z1=xyz[j][2];
      if (((x0-x1)*(x0-x1)+(y0-y1)*(y0-y1)+(z0-z1)*(z0-z1))<0.125) {
	fprintf(stderr,
		"Warning: Atom %d and %d are too close to each other : %f A\n",
		i+1,j+1, 
		sqrt((x0-x1)*(x0-x1)+(y0-y1)*(y0-y1)+(z0-z1)*(z0-z1)));
	/*
	fprintf(stderr,"         Atom %d is merged into %d\n",i+1, j+1);
	atomMap[i]=j;
	break;
	*/
      }
    }
    if (j==k) {
      xyz[k][0]=xyz[i][0];
      xyz[k][1]=xyz[i][1];
      xyz[k][2]=xyz[i][2];
      strcpy(natm[k],natm[i]);
      iTbl[k]=iTbl[i];
      iCol[k]=iCol[i];
      atomMap[i]=k;
      k++;
    }
  }
  nAtom=k;
  for(i=0;i<nExcl;i++){
    iExcl[i]=atomMap[iExcl[i]];
    bExclFlag[iExcl[i]]=1;
  }

  for(j=0;j<nBond;j++) {
    iPs[j]=atomMap[iPs[j]];
    iPe[j]=atomMap[iPe[j]];
  }
  for(i=0;i<nPlt;i++){
    iPlt[i]=atomMap[iPlt[i]];
  }

  free(atomMap);
  return 1;
}


